/*******************************************************************************
 * Copyright (c) 2004 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.core.script;

import com.ibm.icu.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import com.ibm.icu.util.ULocale;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.ScriptableObject;

/**
 * 
 * 
 * The static DateTimeSpan object provides methods to determine the amount of
 * time between two date/times, and to add or subtract time spans. Provides a
 * set of functions for working with the difference between two dates.
 */
public class NativeDateTimeSpan extends ScriptableObject
{

	/**
	 * 
	 */
	private static final long serialVersionUID = 8121313848395430946L;
	/**
	 * logger used to log syntax errors.
	 */
	static protected Logger logger = Logger.getLogger( NativeDateTimeSpan.class
			.getName( ) );

	/**
	 * The zero-argument constructor used by Rhino runtime to create instances
	 */
	public NativeDateTimeSpan( )
	{
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.mozilla.javascript.Scriptable#getClassName()
	 */
	public String getClassName( )
	{
		return "DateTimeSpan";
	}

	/**
	 * convert java script object to double
	 * 
	 * @param value
	 *            js object
	 * @return double value
	 */
	protected static Date toDate( Object value )
	{
		try
		{
			if ( value instanceof String )
			{
				DateFormat date = DateFormat.getDateTimeInstance(
						DateFormat.SHORT, DateFormat.SHORT, ULocale.US );

				return date.parse( (String) value );
			}
			return (Date) Context.toType( value, Date.class );
		}
		catch ( ParseException pe )
		{
			if ( logger.isLoggable( Level.WARNING ) )
			{
				logger.log( Level.WARNING,
						"There exists an error in toDate method. ", pe );
			}
			return null;
		}
	}

	/**
	 * convert java script object to int
	 * 
	 * @param value
	 *            js object
	 * @return int value
	 */
	protected static int toInt( Object value )
	{
		return (int) Context.toNumber( value );
	}

	public static int jsStaticFunction_years( Object start, Object end )
	{
		Date startDate = toDate( start );
		Date endDate = toDate( end );
		return DateTimeSpan.years( startDate, endDate );
	}

	public static int jsStaticFunction_months( Object start, Object end )
	{
		Date startDate = toDate( start );
		Date endDate = toDate( end );
		return DateTimeSpan.months( startDate, endDate );
	}

	public static int jsStaticFunction_days( Object start, Object end )
	{
		Date startDate = toDate( start );
		Date endDate = toDate( end );
		return DateTimeSpan.days( startDate, endDate );
	}

	public static int jsStaticFunction_hours( Object start, Object end )
	{
		Date startDate = toDate( start );
		Date endDate = toDate( end );
		return DateTimeSpan.hours( startDate, endDate );
	}

	public static int jsStaticFunction_minutes( Object start, Object end )
	{
		Date startDate = toDate( start );
		Date endDate = toDate( end );
		return DateTimeSpan.minutes( startDate, endDate );
	}

	public static int jsStaticFunction_seconds( Object start, Object end )
	{
		Date startDate = toDate( start );
		Date endDate = toDate( end );
		return DateTimeSpan.seconds( startDate, endDate );
	}

	public static Object jsStaticFunction_addDate( Object start, int years, int months,
			int days )
	{
		Date startDate = toDate( start );
		return DateTimeSpan.addDate( startDate, years, months, days );
	}

	public static Object jsStaticFunction_addTime( Object start, int hours, int minutes,
			int seconds )
	{
		Date startDate = toDate( start );
		return DateTimeSpan.addTime( startDate, hours, minutes, seconds );
	}

	public static Object jsStaticFunction_subDate( Object start, int years, int months,
			int days )
	{
		Date startDate = toDate( start );
		return DateTimeSpan.subDate( startDate, years, months, days );
	}

	public static Object jsStaticFunction_subTime( Object start, int hours, int minutes,
			int seconds )
	{

		Date startDate = toDate( start );
		return DateTimeSpan.subTime( startDate, hours, minutes, seconds );
	}

}