/*******************************************************************************
 * Copyright (c) 2004, 2008 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation  - initial API and implementation
 *******************************************************************************/

package org.eclipse.birt.data.engine.impl.document;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.birt.core.archive.RAInputStream;
import org.eclipse.birt.core.archive.RAOutputStream;
import org.eclipse.birt.core.util.IOUtil;
import org.eclipse.birt.data.engine.api.DataEngineContext;
import org.eclipse.birt.data.engine.core.DataException;

/**
 * 
 */

public abstract class BaseSave implements IRDSave
{

	protected DataEngineContext context;

	public BaseSave( DataEngineContext context )
	{
		this.context = context;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.birt.data.engine.impl.document.IRDSave#saveNamingRelation(java.util.List)
	 */
	public void saveNamingRelation( NamingRelation relation )
			throws DataException
	{
		Map<String, String> bookmarkMap = new HashMap<String, String>( );
		Map<String, String> elementIdMap = new HashMap<String, String>( );
		loadExistRelations( bookmarkMap, elementIdMap );
		RAOutputStream out = context.getOutputStream( null,
				null,
				DataEngineContext.NAMING_RELATION_STREAM );
		try
		{
			DataOutputStream dos = new DataOutputStream( new BufferedOutputStream( out ) );
			final String rsId = relation.getQueryResultId( );
			if ( relation.getQueryResultName( ) != null )
			{
				bookmarkMap.put( relation.getQueryResultName( ), rsId );
			}
			if ( relation.getQueryDefnName( ) != null )
			{
				elementIdMap.put( relation.getQueryDefnName( ), rsId );
			}
			IOUtil.writeMap( dos, bookmarkMap );
			IOUtil.writeMap( dos, elementIdMap );
			dos.flush( );
		}
		catch ( IOException e )
		{
			throw new DataException( "", e ); //$NON-NLS-1$
		}
		finally
		{
			if ( out != null )
			{
				try
				{
					out.close( );
				}
				catch ( IOException e )
				{
					e.printStackTrace( );
				}
			}
		}
	}

	/**
	 * 
	 * @param bookmarkMap
	 * @param elementIdMap
	 * @throws DataException
	 */
	@SuppressWarnings("unchecked")
	private void loadExistRelations( Map<String, String> bookmarkMap,
			Map<String, String> elementIdMap ) throws DataException
	{
		RAInputStream in = null;
		try
		{
			in = context.getInputStream( null,
					null,
					DataEngineContext.NAMING_RELATION_STREAM );
		}
		catch ( DataException e )
		{
			in = null;
		}

		if ( in != null )
		{
			try
			{
				DataInputStream dis = new DataInputStream( new BufferedInputStream( in ) );
				bookmarkMap.putAll( IOUtil.readMap( dis ) );
				elementIdMap.putAll( IOUtil.readMap( dis ) );
			}
			catch ( IOException e1 )
			{
				e1.printStackTrace( );
			}
			finally
			{
				try
				{
					in.close( );
				}
				catch ( IOException e )
				{
					e.printStackTrace( );
				}
			}
		}
	}

}
