/*************************************************************************************
 * Copyright (c) 2004, 2007 Actuate Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Actuate Corporation - Initial implementation.
 ************************************************************************************/

package org.eclipse.birt.report.data.oda.jdbc.ui.provider;

import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.birt.report.data.oda.jdbc.OdaJdbcDriver;
import org.eclipse.birt.report.data.oda.jdbc.ui.JdbcPlugin;
import org.eclipse.birt.report.data.oda.jdbc.ui.util.ExceptionHandler;
import org.eclipse.datatools.connectivity.oda.IConnection;
import org.eclipse.datatools.connectivity.oda.IDriver;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.design.DataSourceDesign;
import org.eclipse.datatools.connectivity.oda.design.ui.designsession.DesignSessionUtil;
import org.eclipse.ui.PlatformUI;

/**
 * Connection provider to access the IConnection object.
 *
 */
public class OdaConnectionProvider
{

	private IConnection connection;
	private DataSourceDesign dataSourceDesign;
	private static Logger logger = Logger.getLogger( OdaConnectionProvider.class.getName( ) );

	public OdaConnectionProvider( DataSourceDesign dataSourceDesign )
	{
		this.dataSourceDesign = dataSourceDesign;
	}
	
	
	/**
	 * Establish the runtime connection.
	 * @param dataSourceDesign
	 * @return
	 */
	public void connect( ) throws OdaException
	{
		IDriver jdbcDriver = new OdaJdbcDriver( );
		if ( connection != null )
		{
			closeConnection( );
			connection = null;
		}
		connection = jdbcDriver.getConnection( dataSourceDesign.getOdaExtensionId( ) );
		Properties prop = DesignSessionUtil.getEffectiveDataSourceProperties( dataSourceDesign );
		connection.open( prop );
	}

	/**
	 * 
	 * @return
	 */
	public IConnection getConnection( )
	{
		return this.connection;
	}

	/**
	 * Release the connection
	 */
	public void closeConnection( )
	{
		if ( connection != null )
		{
			try
			{
				if ( connection.isOpen( ) )
				{
					connection.close( );
					connection = null;
				}
			}
			catch ( OdaException e )
			{
				logger.log( Level.WARNING, e.getMessage( ), e );
			}
		}
	}
}
